load('R/scDEED/Seurat.RData')
##permuted function from functions.R, with adjustment to change seed
library('aricode')
library(gridExtra)
library(ggsci)
library(RColorBrewer)
library(scales)
library(ROGUE)
Permuted = function (pbmc, seed = 1000) 
{ 
  pbmc.permuted = pbmc
  X=pbmc@assays$RNA@scale.data
  X_permuted = pbmc.permuted@assays$RNA@scale.data
  set.seed(seed)
  for (i in 1:dim(X)[1])
  {
    row = randperm(dim(X)[2])
    X_permuted[i,]=X[i,row]
  }
  pbmc.permuted@assays$RNA@scale.data = X_permuted
  
  return(pbmc.permuted)
}
K = 50
inDrops_permuted = Permuted(inDrops)
inDrops_permuted2 = Permuted(inDrops, seed = 2)

inDrops_permuted<-RunPCA(inDrops_permuted, npcs= K,features = VariableFeatures(object = inDrops_permuted))
inDrops_permuted2<-RunPCA(inDrops_permuted2, npcs= K,features = VariableFeatures(object = inDrops_permuted2))

inDrops <- RunTSNE(inDrops, seed.use=100, perplexity = 40)
inDrops_permuted <- RunTSNE(inDrops_permuted, seed.use=100, perplexity = 40)
inDrops_permuted2 <- RunTSNE(inDrops_permuted2, seed.use=100, perplexity = 40)
test = CreateSeuratObject(inDrops_permuted@assays$RNA@scale.data)

plot(inDrops_permuted@reductions$tsne@cell.embeddings)

p1 = DimPlot(inDrops, group.by = 'CellType', reduction = 'tsne')
p2 = DimPlot(inDrops_permuted, group.by = 'CellType', reduction = 'tsne')
p3 = DimPlot(inDrops_permuted2, group.by = 'CellType',reduction = 'tsne')

p = grid.arrange(p1, p2, p3, ncol = 3)
ggsave(p, filename = '/home/christy/R/scDEED/permuted_clusters.pdf', width = 12, height = 2.5)

p1 = DimPlot(inDrops, reduction = 'pca',  group.by = 'CellType')
p2 = DimPlot(inDrops_permuted, reduction = 'pca',group.by = 'CellType')
p3 = DimPlot(inDrops_permuted2, reduction = 'pca',group.by = 'CellType')

p = grid.arrange(p1, p2, p3, ncol = 3)
ggsave(p, filename = '/home/christy/R/scDEED/permuted_clusters_pca.pdf', width = 12, height = 2.5)


###heatmaps######

B_cell = which(inDrops$CellType=='B cell')
CD14 = which(inDrops$CellType=='CD14+ monocyte')
cd4 = which(inDrops$CellType == 'CD4+ T cell')
cyto = which(inDrops$CellType == 'Cytotoxic T cell')

##colors, b cell = #F8766D, cd14 = #7CAE00, cd4 = #00BFC4, cytotoxic = #C77CFF
show_col(hue_pal()(4))
X=inDrops@assays$RNA@scale.data
X= t(X)
sum(X>1)/length(as.vector(X))
#0.02311 of the cells have expression values greater than 1; we threshold this to make a more meaningful heatmap
X[X>1] = 1
X = rbind(X[B_cell, ],X[CD14,],X[cd4,], X[cyto, ])
selected_genes = sort(colVars(X),decreasing=TRUE,index.return=TRUE)$ix[1:300]

mypal <- c(pal_material(palette="blue",alpha = 0.8,reverse=TRUE,n=12)(12),pal_material(palette="orange",alpha = 0.8,n=12)(12))
length(mypal)
### C Heatmap for neuron ec1 dubious embeddings vs. trustworthy, together with sc
rowSide <- c(rep('#F8766D', length(B_cell)), rep('#7CAE00', length(CD14)), rep('#00BFC4', length(cd4)), rep('#C77CFF', length(cyto)))
plotname<-"inDrops_heatmap.png"
png(filename = plotname, width = 10, height = 10, units = "in", res = 500)
par(oma=c(0,0,0,0))
heatmap.2(X[,selected_genes], dendrogram="none",density.info ="none",RowSideColors=rowSide,col=mypal,trace="none",labRow=NA,rowV = F,Colv=NA,Rowv=NA, labCol=NA,key=T,  breaks=seq(-1,1,length = 25), symm=F,symkey=F,symbreaks=T, scale="none")
mtext("genes",side = 1,line =2.8, adj = 0.6,cex=2)
mtext("cells",side = 2, line = -3,adj = 0.3,cex=2)
dev.off()


X=inDrops_permuted@assays$RNA@scale.data
X= t(X)
sum(X>1)/length(as.vector(X))
#0.02311 of the cells have expression values greater than 1; we threshold this to make a more meaningful heatmap
X[X>1] = 1
X = rbind(X[B_cell, ],X[CD14,],X[cd4,], X[cyto, ])
selected_genes = sort(colVars(X),decreasing=TRUE,index.return=TRUE)$ix[1:300]
max(X)
min(X)
mypal <- c(pal_material(palette="blue",alpha = 0.8,reverse=TRUE,n=12)(12),pal_material(palette="orange",alpha = 0.8,n=12)(12))
length(mypal)
### C Heatmap for neuron ec1 dubious embeddings vs. trustworthy, together with sc
rowSide <- c(rep('#F8766D', length(B_cell)), rep('#7CAE00', length(CD14)), rep('#00BFC4', length(cd4)), rep('#C77CFF', length(cyto)))
plotname<-"inDrops_permuted.png"
png(filename = plotname, width = 10, height = 10, units = "in", res = 500)
par(oma=c(0,0,0,0))
heatmap.2(X[,selected_genes], dendrogram="none",density.info ="none",RowSideColors=rowSide,col=mypal,trace="none",labRow=NA,rowV = F,Colv=NA,Rowv=NA, labCol=NA,key=T,  breaks=seq(-1,1,length = 25), symm=F,symkey=F,symbreaks=T, scale="none")
mtext("genes",side = 1,line =2.8, adj = 0.6,cex=2)
mtext("cells",side = 2, line = -3,adj = 0.3,cex=2)
dev.off()

X=inDrops_permuted2@assays$RNA@scale.data
X= t(X)
sum(X>1)/length(as.vector(X))
#0.02311 of the cells have expression values greater than 1; we threshold this to make a more meaningful heatmap
X[X>1] = 1
X = rbind(X[B_cell, ],X[CD14,],X[cd4,], X[cyto, ])
selected_genes = sort(colVars(X),decreasing=TRUE,index.return=TRUE)$ix[1:300]
max(X)
min(X)
mypal <- c(pal_material(palette="blue",alpha = 0.8,reverse=TRUE,n=12)(12),pal_material(palette="orange",alpha = 0.8,n=12)(12))
length(mypal)
### C Heatmap for neuron ec1 dubious embeddings vs. trustworthy, together with sc
rowSide <- c(rep('#F8766D', length(B_cell)), rep('#7CAE00', length(CD14)), rep('#00BFC4', length(cd4)), rep('#C77CFF', length(cyto)))
plotname<-"inDrops_permuted2.png"
png(filename = plotname, width = 10, height = 10, units = "in", res = 500)
par(oma=c(0,0,0,0))
heatmap.2(X[,selected_genes], dendrogram="none",density.info ="none",RowSideColors=rowSide,col=mypal,trace="none",labRow=NA,rowV = F,Colv=NA,Rowv=NA, labCol=NA,key=T,  breaks=seq(-1,1,length = 25), symm=F,symkey=F,symbreaks=T, scale="none")
mtext("genes",side = 1,line =2.8, adj = 0.6,cex=2)
mtext("cells",side = 2, line = -3,adj = 0.3,cex=2)
dev.off()
