
# Permutation for input as Seurat object
Permuted = function (pbmc) 
{ 
    pbmc.permuted = pbmc
    X=pbmc@assays$RNA@scale.data
    X_permuted = pbmc.permuted@assays$RNA@scale.data
    set.seed(1000)
for (i in 1:dim(X)[1])
{
    row = randperm(dim(X)[2])
    X_permuted[i,]=X[i,row]
}
pbmc.permuted@assays$RNA@scale.data = X_permuted
    
    return(pbmc.permuted)
}


# Permutation for matrix input
Permuted.matrix = function (X) 
{ 
  X_permuted = X
  set.seed(1000)
  for (i in 1:dim(X)[2])
  {
    column = randperm(dim(X)[1])
    X_permuted[,i]=X[column,i]
  }
  return(X_permuted)
}


PermutedICA = function (pbmc) 
{ 
  pbmc.permuted = pbmc
  X=pbmc@reductions$ica@cell.embeddings
  X_permuted = pbmc.permuted@reductions$ica@cell.embeddings
  set.seed(1000)
  for (i in 1:dim(X)[2])
  {
    column = randperm(dim(X)[1])
    X_permuted[,i]=X[column,i]
  }
  pbmc.permuted@reductions$ica@cell.embeddings = X_permuted
  return(pbmc.permuted)
}

# Find distances under PCA and tSNE for both original and permuted matrices.
Distances.PCA.big = function(pbmc,pbmc.permuted, K, perplexity_score)
{

  distances<-distances::distances
   pbmc.permuted<-RunPCA(pbmc.permuted, npcs= K,features = VariableFeatures(object = pbmc.permuted))
  
  
  M=pbmc@reductions[["pca"]]@cell.embeddings
  PCA_distances = distances(M[,1:K])
  
  M_permuted=pbmc.permuted@reductions[["pca"]]@cell.embeddings
  PCA_distances_permuted = distances(M_permuted[,1:K])
  

  tSNE_distances = distances(pbmc@reductions$tsne@cell.embeddings)
  
  pbmc.permuted <- RunTSNE(pbmc.permuted, dims.use = 1:K, seed.use=100, perplexity = perplexity_score)
  tSNE_distances_permuted = distances(pbmc.permuted@reductions$tsne@cell.embeddings)
  
  results.PCA<-list("object"=pbmc,"object.permuted"=pbmc.permuted,"PCA_distances"=PCA_distances,"PCA_distances_permuted"=PCA_distances_permuted,"tSNE_distances"=tSNE_distances,"tSNE_distances_permuted"=tSNE_distances_permuted)
  
  return(results.PCA)
}

# Find distances under PCA and UMAP for both original and permuted matrices.
Distances.PCA.UMAP.big = function(pbmc,pbmc.permuted, K)
{

  distances<-distances::distances
  pbmc.permuted<-RunPCA(pbmc.permuted, npcs= K,features = VariableFeatures(object = pbmc.permuted))


  M=pbmc@reductions[["pca"]]@cell.embeddings
  PCA_distances = distances(M[,1:K])

  M_permuted=pbmc.permuted@reductions[["pca"]]@cell.embeddings
  PCA_distances_permuted = distances(M_permuted[,1:K])


  UMAP_distances = distances(pbmc@reductions$umap@cell.embeddings)

  pbmc.permuted <- RunUMAP(pbmc.permuted, dims = 1:K, seed.use=100)
  UMAP_distances_permuted = distances(pbmc.permuted@reductions$umap@cell.embeddings)

  results.PCA<-list("object"=pbmc,"object.permuted"=pbmc.permuted,"PCA_distances"=PCA_distances,"PCA_distances_permuted"=PCA_distances_permuted,"UMAP_distances"=UMAP_distances,"UMAP_distances_permuted"=UMAP_distances_permuted)

  return(results.PCA)
}

Distances.PCA.tSNE.big = function(pbmc,pbmc.permuted, K, perplexity_score)
{
  
  distances<-distances::distances
  pbmc.permuted<-RunPCA(pbmc.permuted, npcs= K,features = VariableFeatures(object = pbmc.permuted))
  
  
  M=pbmc@reductions[["pca"]]@cell.embeddings
  PCA_distances = distances(M[,1:K])
  
  M_permuted=pbmc.permuted@reductions[["pca"]]@cell.embeddings
  PCA_distances_permuted = distances(M_permuted[,1:K])
  
  pbmc <- RunTSNE(pbmc, dims.use = 1:K, seed.use=100, perplexity = perplexity_score)
  tSNE_distances = distances(pbmc@reductions$tsne@cell.embeddings)
  
  pbmc.permuted <- RunTSNE(pbmc.permuted, dims.use = 1:K, seed.use=100, perplexity = perplexity_score)
  tSNE_distances_permuted = distances(pbmc.permuted@reductions$tsne@cell.embeddings)
  
  results.PCA<-list("object"=pbmc,"object.permuted"=pbmc.permuted,"PCA_distances"=PCA_distances,"PCA_distances_permuted"=PCA_distances_permuted,"tSNE_distances"=tSNE_distances,"tSNE_distances_permuted"=tSNE_distances_permuted)
  
  return(results.PCA)
}


 Distances.ICA = function(pbmc,pbmc.permuted,K,n,mindist)
{

  distances<-distances::distances
  ICA_distances_permuted = distances(pbmc.permuted@reductions$ica@cell.embeddings)

  pbmc.permuted <- RunUMAP(pbmc.permuted,   reduction = "ica", dims = 1:K, seed.use=100, n.neighbors = n,min.dist=mindist)
  UMAP_distances_permuted = distances(pbmc.permuted@reductions$umap@cell.embeddings)

  results.ICA<-list("object.permuted"=pbmc.permuted,"ICA_distances_permuted"=ICA_distances_permuted,"UMAP_distances_permuted"=UMAP_distances_permuted)

  return(results.ICA)
 }
 


# Choose optimal perplexity parameter under tSNE. Here for one parameter
ChoosePerplexity<- function(pbmc,pbmc.permuted, K, perplexity_score){
  distances<-distances::distances
   pbmc_perplex = RunTSNE(pbmc, dims.use = 1:K, seed.use = 100, perplexity = perplexity_score, do.fast = T)
    print("tSNE done")
    tSNE_distances = distances(pbmc_perplex@reductions$tsne@cell.embeddings)
    pbmc.permuted <- RunTSNE(pbmc.permuted, dims.use = 1:K, seed.use = 100, perplexity = perplexity_score, do.fast = T)
      print("permuted tSNE done")
    tSNE_distances_permuted = distances(pbmc.permuted@reductions$tsne@cell.embeddings)
    results.PCA<-list("object"= pbmc_perplex,"object.permuted"=pbmc.permuted,"tSNE_distances"=tSNE_distances,"tSNE_distances_permuted"=tSNE_distances_permuted)
        return(results.PCA)
}

ChoosePerplexity.ICA<- function(pbmc,pbmc.permuted, K, perplexity_score){
  distances<-distances::distances
  pbmc_perplex = RunTSNE(pbmc, dims.use = 1:K, reduction="ica", seed.use = 100, perplexity = perplexity_score, do.fast = T)
  print("tSNE done")
  tSNE_distances = distances(pbmc_perplex@reductions$tsne@cell.embeddings)
  pbmc.permuted <- RunTSNE(pbmc.permuted, dims.use = 1:K, reduction="ica", seed.use = 100, perplexity = perplexity_score, do.fast = T)
  print("permuted tSNE done")
  tSNE_distances_permuted = distances(pbmc.permuted@reductions$tsne@cell.embeddings)
  results.ICA<-list("object"= pbmc_perplex,"object.permuted"=pbmc.permuted,"tSNE_distances"=tSNE_distances,"tSNE_distances_permuted"=tSNE_distances_permuted)
  return(results.ICA)
}


# Choose the min.dist parameter in UMAP.
ChooseUMAP<- function(pbmc,pbmc.permuted, reduction.method, K, n, mindist){
  distances<-distances::distances
  pbmc_para = RunUMAP(pbmc, reduction = reduction.method, dims = 1:K, seed.use = 100, n.neighbors=n, min.dist=mindist, do.fast = T)
  print("UMAP done")
  UMAP_distances = distances(pbmc_para@reductions$umap@cell.embeddings)
  pbmc.permuted <- RunUMAP(pbmc.permuted, reduction = reduction.method, dims = 1:K, seed.use = 100, n.neighbors=n, min.dist=mindist, do.fast = T)
  print("permuted UMAP done")
  UMAP_distances_permuted = distances(pbmc.permuted@reductions$umap@cell.embeddings)
  results.PCA<-list("object"= pbmc_para,"object.permuted"=pbmc.permuted,"UMAP_distances"=UMAP_distances,"UMAP_distances_permuted"=UMAP_distances_permuted)
  return(results.PCA)
}



# Choose the n.neighbors parameter in UMAP. Here for one parameter
ChoosenNeighbors<- function(pbmc,pbmc.permuted, reduction.method, K, n){
  distances<-distances::distances
  pbmc_nNeighbors = RunUMAP(pbmc, reduction = reduction.method,dims = 1:K, seed.use = 100, n.neighbors=n, do.fast = T)
  print("UMAP done")
  UMAP_distances = distances(pbmc_nNeighbors@reductions$umap@cell.embeddings)
  pbmc.permuted <- RunUMAP(pbmc.permuted, reduction = reduction.method, dims = 1:K, seed.use = 100, n.neighbors=n, do.fast = T)
  print("permuted UMAP done")
  UMAP_distances_permuted = distances(pbmc.permuted@reductions$umap@cell.embeddings)
  results<-list("object"= pbmc_nNeighbors,"object.permuted"=pbmc.permuted,"UMAP_distances"=UMAP_distances,"UMAP_distances_permuted"=UMAP_distances_permuted)
  return(results)
}



Cell.Similarity.UMAP = function(Euc_distances,Euc_distances_permuted,UMAP_distances,UMAP_distances_permuted,percent)
    {
  numberselected = floor((dim(Euc_distances)[2])*percent)
  rho_UMAP<-foreach(i=1:(dim(Euc_distances)[2]))%dopar%{
    rho_UMAP=cor((UMAP_distances[i,order(Euc_distances[i,])][2:(numberselected+1)]),(sort(UMAP_distances[i,])[2:(numberselected+1)]))
  }
  rho_UMAP = as.numeric(rho_UMAP)
  print("UMAP done")
  rho_UMAP_permuted<-foreach(i=1:(dim(Euc_distances)[2]))%dopar%{
    rho_UMAP_permuted=cor((UMAP_distances_permuted[i,order(Euc_distances_permuted[i,])][2:(numberselected+1)]),(sort(UMAP_distances_permuted[i,])[2:(numberselected+1)]))
  }
  rho_UMAP_permuted = as.numeric(rho_UMAP_permuted)
  print("permuted UMAP done")
  similarity_results_UMAP <-list("rho_UMAP"=rho_UMAP,"rho_UMAP_permuted"=rho_UMAP_permuted)
  return(similarity_results_UMAP)
}


Cell.Similarity.tSNE = function(Euc_distances,Euc_distances_permuted,tSNE_distances,tSNE_distances_permuted,percent)
{
    numberselected = floor((dim(Euc_distances)[2])*percent)
    
    rho_tSNE<-foreach(i=1:(dim(Euc_distances)[2]))%dopar%{
    rho_tSNE=cor((tSNE_distances[i,order(Euc_distances[i,])][2:(numberselected+1)]),(sort(tSNE_distances[i,])[2:(numberselected+1)]))
        }
    rho_tSNE = as.numeric(rho_tSNE)
    print("tSNE done")
    rho_tSNE_permuted<-foreach(i=1:(dim(Euc_distances)[2]))%dopar%{
           rho_tSNE_permuted=cor((tSNE_distances_permuted[i,order(Euc_distances_permuted[i,])][2:(numberselected+1)]),(sort(tSNE_distances_permuted[i,])[2:(numberselected+1)]))
         }
    rho_tSNE_permuted = as.numeric(rho_tSNE_permuted)
    print("permuted tSNE done")
       similarity_results_tSNE <-list("rho_tSNE"=rho_tSNE,"rho_tSNE_permuted"=rho_tSNE_permuted)
    return(similarity_results_tSNE)
}



Cell.Classify.tSNE = function(rho_tSNE,rho_tSNE_permuted){
    y = seq(1,length(rho_tSNE),length=length(rho_tSNE))

    rho_tSNE_upper = quantile(rho_tSNE_permuted,0.95)
    rho_tSNE_lower = quantile(rho_tSNE_permuted,0.05)
    
  
    tSNE_badindex = which(rho_tSNE < rho_tSNE_lower)
    tSNE_intindex = which(rho_tSNE< rho_tSNE_upper&rho_tSNE >= rho_tSNE_lower)
    tSNE_goodindex = setdiff(y,union(tSNE_badindex,tSNE_intindex))
    ClassifiedCells_tSNE.results<-list("tSNE_badindex"=tSNE_badindex,"tSNE_intindex"=tSNE_intindex,"tSNE_goodindex"=tSNE_goodindex)
    return(ClassifiedCells_tSNE.results)
}

Cell.Classify.UMAP = function(rho_UMAP,rho_UMAP_permuted){
    y = seq(1,length(rho_UMAP),length=length(rho_UMAP))
    rho_UMAP_upper = quantile(rho_UMAP_permuted,0.95)
    rho_UMAP_lower = quantile(rho_UMAP_permuted,0.05)
    
    UMAP_badindex = which(rho_UMAP < rho_UMAP_lower)
    UMAP_intindex = which(rho_UMAP< rho_UMAP_upper&rho_UMAP >= rho_UMAP_lower)
    UMAP_goodindex = setdiff(y,union(UMAP_badindex,UMAP_intindex))
    
   ClassifiedCells_UMAP.results<-list("UMAP_badindex"=UMAP_badindex,"UMAP_intindex"=UMAP_intindex,"UMAP_goodindex"=UMAP_goodindex)
    return(ClassifiedCells_UMAP.results)
}